%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%   Equilibrium Computation Code   	%%%%%%%%%%%%
%%%%%%%%%%%%   								    %%%%%%%%%%%%
%%%%%%%%%%%%									%%%%%%%%%%%%
%%%%%%%%%%%%   Handel, Hendel, Whinston         %%%%%%%%%%%%
%%%%%%%%%%%%   handel@berkeley.edu              %%%%%%%%%%%%
%%%%%%%%%%%%									%%%%%%%%%%%%
%%%%%%%%%%%%   Equilibria in Health Exchanges   %%%%%%%%%%%%
%%%%%%%%%%%%   Adverse Selection vs. 	        %%%%%%%%%%%%
%%%%%%%%%%%%   Reclassification Risk            %%%%%%%%%%%%
%%%%%%%%%%%%                                    %%%%%%%%%%%%
%%%%%%%%%%%%   January, 2015               	    %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% Computes numbers that are used to compute the Equilbirium in Excel.

%%% NOTE: This is not set up to be used with simulated data provided, since there are several steps in between simulated data provided and 
%%% getting to this counterfactual market analysis in terms of data processing. 

%% ESAge is age for this sample
%% RPInd is matrix of risk preferences from estimation
%% ESTOTCR_ is matrix describing individual expected health risk 
%% EUVector is vector of expected utility wealth values for each plan and state of the world 
%% SampleSize1 is size of sample
%% Sim is number of health risk and risk preference simulations 
%% ExchPlanPaid is amount paid by plan for each individual in each state of the world.  

load 'EqCalcInput.mat' RPInd EUvector ESTOTCRindex_UN SampleSize1 Sim Exchplanpaid ESage


AgeG=25;


%%%% Number of ACG groups: Here example if for N = 4 groups (same example as in Section 4 of paper)
N=4;
Grid=0:1/N:1;

percentile=quantile(ESTOTCRindex_UN(ESage>=AgeG & ESage<AgeG+5,1),Grid);

% Different Quartiles.
data{N+1}=[];

for iter=4
    
    %All
    clearvars -except iter RPInd EUvector ESTOTCRindex_UN SampleSize1...
        Sim Exchplanpaid ESage data percentile N AgeG
        
    % ALL
    if iter==1
        al= -100;  ah=110;        
    else        
        al=percentile(iter-1); ah=percentile(iter);        
    end
    
    % Setting equilibrium variables
    pr4=zeros(6,1); pr1=pr4; sh1=pr4; sh4=pr4; ac1=sh1; ac4=sh4; ddp=sh1;
    pr12=pr1; pr42=pr4; i=0;
    %Select price range
    ddp=-3000:50:8000;
    
    % Restrict Sample to the corresponding AGE bin (this is for analysis that conditions on age as well as health risk in market formation)
    a=((ESTOTCRindex_UN>al).*(ESTOTCRindex_UN<=ah).*(ESage>=AgeG).*(ESage<AgeG+5));
    aa=ones(SampleSize1,1);
    
    % QUARTILE-SIZE by SIM matrix with different price possibilities
    dp= permute(squeeze( repmat( ddp , [1,1,sum(a),Sim] )) , [2,3,1] );
    
    % Cost matrices for this quartile
    
    C1=repmat( reshape(Exchplanpaid(a>0,1,:),sum(a),Sim) , [1,1,length(ddp)]);
    C2=repmat( reshape(Exchplanpaid(a>0,2,:),sum(a),Sim) , [1,1,length(ddp)]);
    C3=repmat( reshape(Exchplanpaid(a>0,3,:),sum(a),Sim) , [1,1,length(ddp)]);
    C4=repmat( reshape(Exchplanpaid(a>0,4,:),sum(a),Sim) , [1,1,length(ddp)]);
    
    % ExpectedUtilities for each person, plan, type of health risk
    
    EU1=repmat( reshape(EUvector(a>0,1,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU2=repmat( reshape(EUvector(a>0,2,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU3=repmat( reshape(EUvector(a>0,3,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU4=repmat( reshape(EUvector(a>0,4,:),sum(a),Sim) , [1,1,length(ddp)]);
    
    % Risk preference indicies
    
    RPI=repmat( RPInd(a>0,:) , [1,1,length(ddp)]);
    
    %Below is in case we want to put in error matrices
    
    %ShockSize
    sd=525;
    
    prob90 =cdf('norm', (log(EU1./EU4))./RPI - dp, 0,sd) ;
    prob90=prob90.*(prob90<=0.999).*(prob90>=0.0001) + (prob90>0.999);
    
    
    s1=sum(squeeze(sum(1-prob90,1)),1)+1e-10;
    s4=sum(squeeze(sum(prob90,1)),1)+1e-10;
    
    sh1=s1/(sum(a)*Sim); %This is in [0,1]
    sh4=s4/(sum(a)*Sim);
    
    ac1=sum(squeeze(sum((1-prob90).*C1,1)),1)./s1;
    ac4=sum(squeeze(sum((prob90).*C4,1)),1)./s4;
    
    
    %Difference in average costs for each different potential relative price
    dac=ac4-ac1;
    
    %Total costs and
    
    % Given ddp, p60be is the price of plan 60 that makes total industry profits
    % equal to zero.
    p60be=(sh1.*ac1+sh4.*ac4-ddp.*sh4);
    
    
    % By definition, the price of plan 90
    p90be=p60be+ddp;
    
    % Profits of the firm offering plan 90.
    pr90=(p90be-ac4).*sh4;
    
    % If only 1 firm was offering plan 60, the profits of an entrant offering
    % plan 90, when the incumbent does not change the price :
    
    
    p90=mean(C4(:,1,1));
    % Profits of the deviation 'Offer 60' when only 90 is currently
    % offered.
    e90 = sum(squeeze(sum(prob90.*(abs(p90-dp)-C1),1)),1)*1e-6;
    
    sh1=sh1*100;
    sh4=sh4*100;
        
    data{iter,AgeG}=[ddp' dac' pr90' p60be' p90be' ac1' ac4' sh1' sh4' e90'];  
    dat=[ddp' dac' pr90' p60be' p90be' ac1' ac4' sh1' sh4' e90']
end

%%%%%%%%%%%%%%%%%%%%% Take output matrices with delta AC as function of Delta P for each market, and compute equilibria following proposition 1, using excel spreadsheet. 

