%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%   Welfare Computation Code 	  	%%%%%%%%%%%%
%%%%%%%%%%%%   CEQs of Exchange Outcomes 		%%%%%%%%%%%%
%%%%%%%%%%%%   GIVEN EQUILIBRIUM OUTCOME 	  	%%%%%%%%%%%%
%%%%%%%%%%%%   								    %%%%%%%%%%%%
%%%%%%%%%%%%									%%%%%%%%%%%%
%%%%%%%%%%%%   Handel, Hendel, Whinston         %%%%%%%%%%%%
%%%%%%%%%%%%   handel@berkeley.edu              %%%%%%%%%%%%
%%%%%%%%%%%%									%%%%%%%%%%%%
%%%%%%%%%%%%   Equilibria in Health Exchanges   %%%%%%%%%%%%
%%%%%%%%%%%%   Adverse Selection vs. 	        %%%%%%%%%%%%
%%%%%%%%%%%%   Reclassification Risk            %%%%%%%%%%%%
%%%%%%%%%%%%                                    %%%%%%%%%%%%
%%%%%%%%%%%%   January, 2015               	    %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Computes wlfare based on equilibrium outcomes computed in Equilibrium_HHW file

%%% NOTE: This is not set up to be used with simulated data provided, since there are several steps in between simulated data provided and 
%%% getting to this counterfactual market analysis in terms of data processing. 

%% ESAge is age for this sample
%% RPInd is matrix of risk preferences from estimation
%% ESTOTCR_ is matrix describing individual expected health risk 
%% EUVector is vector of expected utility wealth values for each plan and state of the world 
%% SampleSize1 is size of sample
%% Sim is number of health risk and risk preference simulations 
%% ExchPlanPaid is amount paid by plan for each individual in each state of the world. 

load 'EqCalcInput.mat' RPInd EUvector ESTOTCRindex_UN SampleSize1 Sim Exchplanpaid ESage
    al= 0; ah=25; 
    
    % Setting equilibrium variables
    pr4=zeros(6,1); pr1=pr4; sh1=pr4; sh4=pr4; ac1=sh1; ac4=sh4; ddp=sh1; pr12=pr1; pr42=pr4; i=0; sd=525; e1=0; e4=0; 
    %Select price range
    ddp=600:200:6200;

    % Restrict Sample to ACG Quartile we are interested and appropriate age range
    a=((ESTOTCRindex_UN>al).*(ESTOTCRindex_UN<=ah).*(ESage>=24.99).*(ESage<65.01));
    aa=ones(SampleSize1,1);

    % This seems to be a QUARTILE-SIZE by SIM matrix with different price possibilities
    dp= permute(squeeze( repmat( ddp , [1,1,sum(a),Sim] )) , [2,3,1] );

    % Cost matrices for this quartile 
    C1=repmat( reshape(Exchplanpaid(a>0,1,:),sum(a),Sim) , [1,1,length(ddp)]);
    C2=repmat( reshape(Exchplanpaid(a>0,2,:),sum(a),Sim) , [1,1,length(ddp)]);
    C3=repmat( reshape(Exchplanpaid(a>0,3,:),sum(a),Sim) , [1,1,length(ddp)]);
    C4=repmat( reshape(Exchplanpaid(a>0,4,:),sum(a),Sim) , [1,1,length(ddp)]);

    % ExpectedUtilitiesTimes100 without price and not negative
    EU1=repmat( reshape(EUvector(a>0,1,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU2=repmat( reshape(EUvector(a>0,2,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU3=repmat( reshape(EUvector(a>0,3,:),sum(a),Sim) , [1,1,length(ddp)]);
    EU4=repmat( reshape(EUvector(a>0,4,:),sum(a),Sim) , [1,1,length(ddp)]);

    % Risk preference indicies
    e1t=randn(size(RPInd));
    RPI=repmat( RPInd(a>0,:) , [1,1,length(ddp)]);
    e1=repmat( e1t(a>0,:) , [1,1,length(ddp)]);

    % Now create expected utilities as a function of price for as many plans as we want 
    EUvectorIT = zeros(sum(a),2,Sim,length(ddp));

    EUvectorIT(:,1,:,:) = EU1.*exp(RPI.*(1000+sd*e1) );
    EUvectorIT(:,2,:,:) = EU4.*exp(RPI.*(1000+dp) ) ;

    % Avg costs in each plan if everyone enrolled in one of the two plans
    p90=mean(C4(:,1,1))
    p60=mean(C1(:,1,1))
    p60std=std(C1(:,1,1))
    p90std=std(C4(:,1,1))

    % Choices that are made as a function of dp
    [C, ConsChoiceI] = min(EUvectorIT,[],2);
    ConsChoice = reshape(ConsChoiceI(:,1,:,:),sum(a),Sim,length(ddp));

    % Number of people in each plan (each real person is simulated 100 times)
    s1=sum( squeeze( sum( (ConsChoice==1),1)),1);
    s4=sum( squeeze( sum( (ConsChoice==2),1)),1);

    % Percentage of the market in each plans given ddp
    sh1=s1/sum(a);
    sh4=s4/sum(a);

    % Firm's average costs
    ac1=sum( squeeze( sum( (ConsChoice==1).*C1,1)),1)./(s1+.0001);
    ac4=sum( squeeze( sum( (ConsChoice==2).*C4,1)),1)./(s4+0.001);

    %Difference in average costs
    dac=ac4-ac1;
    hold off
    plot(ddp,dac)
    hold 
    plot(ddp,ddp)
    
    % Given ddp, p60be is the price of plan 60 that makes total industry profits 
    % equal to zero. Nomalize sh by 100 to get shares in [0,1].
    p60be=(sh1.*ac1+sh4.*ac4-ddp.*sh4)/100;
    % By definition, the price of plan 90
    p90be=p60be+ddp;

    % Profits of the firm offering plan 90.
    pr90=(p90be-ac4).*sh4/100;

    % If only 1 firm was offering plan 60, the profits of an entrant offering
    % plan 90, when the incumbent does not change the price :
    e90 = sum(squeeze( sum((ConsChoice==1).*(abs(p90-dp)-C1),1) ),1 )*1e-6;

    % Data to import into excel and figure out wilson and Nash
    dataall=[ddp' dac' pr90' p60be' p90be' ac1' ac4' sh1' sh4' e90'];
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60+sd*e1t)) )<squeeze(EUvector(:,2,:));
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
    p60it=sum(sum(squeeze(C1(:,:,1)).*in(a>0,:)))/sum(sum(in(a>0,:)))
    in=(squeeze(EUvector(:,1,:)).*exp(RPInd.*(p60it+sd*e1t)) )<squeeze(EUvector(:,2,:));
    mean(mean(in))
   
   %%%% Compute certainty equivalents for each consumer in each plan and each candidate equilibrium price 
   
    CEQ1 = 75000+(log(squeeze(EUvector(a>0,1,:)))-100)./RPInd(a>0,:) +p60;
    CEQ0 = 75000+(log(squeeze(EUvector(a>0,2,:)))-100)./RPInd(a>0,:) ;
    CEQ4 = 75000+(log(squeeze(EUvector(a>0,4,:)))-100)./RPInd(a>0,:) +p90;
    CEQ1nm = 75000+(log(squeeze(EUvector(a>0,1,:)))-100)./RPInd(a>0,:) +p60it;
 
	%%%% Use these certainty equivalent as inputs into long-run welfare calculation. Since we cannot provide actual data, 
	%%%% we refer the reader to the main draft text for the details of the long-run computation. If there are any questions 
	%%%% about how to implement that, please contact the authors at handel@berkeley.edu and we will help, including 
	%%%% supplying extra code for the different long-run models, if useful. 
 